@@ -8,7 +8,7 @@
 
 #include <asm/types.h>          /* For __uXX types */
 
-#define IP_VS_VERSION_CODE            0x010002
+#define IP_VS_VERSION_CODE            0x010003
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -921,10 +921,16 @@ static inline void ip_vs_fast_check_update(union ip_vs_tphdr *h,
 }
 
 static inline int
-ip_vs_skb_cow(struct sk_buff *skb, struct iphdr **iph_p, unsigned char **t_p)
+ip_vs_skb_cow(struct sk_buff *skb, unsigned int headroom,
+	      struct iphdr **iph_p, unsigned char **t_p)
 {
-	if (skb_cloned(skb)) {
-		if (pskb_expand_head(skb, 0, 0, GFP_ATOMIC))
+	int delta = (headroom > 16 ? headroom : 16) - skb_headroom(skb);
+
+	if (delta < 0)
+		delta = 0;
+
+	if (delta ||skb_cloned(skb)) {
+		if (pskb_expand_head(skb, (delta+15)&~15, 0, GFP_ATOMIC))
 			return -ENOMEM;
 
 		/* skb data changed, update pointers */
