@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.25 2002/01/31 14:33:12 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.26 2002/05/30 13:00:20 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -628,6 +628,17 @@ static int ip_vs_bypass_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 		goto tx_error;
 	}
 
+	if (skb_is_nonlinear(skb) && skb->len <= mtu)
+		ip_send_check(iph);
+
+	if (unlikely(skb_headroom(skb) < rt->u.dst.dev->hard_header_len)) {
+		if (skb_cow(skb, rt->u.dst.dev->hard_header_len)) {
+			ip_rt_put(rt);
+			IP_VS_ERR_RL("ip_vs_bypass_xmit(): no memory\n");
+			goto tx_error;
+		}
+	}
+
 	/* drop old route */
 	dst_release(skb->dst);
 	skb->dst = &rt->u.dst;
@@ -741,7 +752,7 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	skb->dst = &rt->u.dst;
 
 	/* copy-on-write the packet before mangling it */
-	if (ip_vs_skb_cow(skb, &iph, &h.raw))
+	if (ip_vs_skb_cow(skb, rt->u.dst.dev->hard_header_len, &iph, &h.raw))
 		return NF_DROP;
 
 	/* mangle the packet */
@@ -967,6 +978,14 @@ static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	if (skb_is_nonlinear(skb) && skb->len <= mtu)
 		ip_send_check(iph);
 
+	if (unlikely(skb_headroom(skb) < rt->u.dst.dev->hard_header_len)) {
+		if (skb_cow(skb, rt->u.dst.dev->hard_header_len)) {
+			ip_rt_put(rt);
+			IP_VS_ERR_RL("ip_vs_dr_xmit(): no memory\n");
+			goto tx_error;
+		}
+	}
+
 	/* drop old route */
 	dst_release(skb->dst);
 	skb->dst = &rt->u.dst;
