@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.29 2002/03/25 12:44:35 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.30 2002/05/30 13:00:20 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -944,8 +944,28 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 		return ret;
 	}
 
+	/*
+	 * mangle and send the packet here
+	 */
+	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(iph->tos))))
+		goto tx_error_icmp;
+
+	/* MTU checking */
+	mtu = rt->u.dst.pmtu;
+	if ((skb->len > mtu) && (iph->frag_off&__constant_htons(IP_DF))) {
+		ip_rt_put(rt);
+		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
+		IP_VS_DBG_RL("ip_vs_in_icmp(): frag needed\n");
+		goto tx_error;
+	}
+
+	/* drop old route */
+	dst_release(skb->dst);
+	skb->dst = &rt->u.dst;
+
 	/* copy-on-write the packet before mangling it */
-	if (ip_vs_skb_cow(skb, &iph, (unsigned char**)&icmph)) {
+	if (ip_vs_skb_cow(skb, rt->u.dst.dev->hard_header_len,
+			  &iph, (unsigned char**)&icmph)) {
 		ip_vs_conn_put(cp);
 		return NF_DROP;
 	}
@@ -976,25 +996,6 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 		  NIPQUAD(ciph->saddr), ntohs(pptr[0]),
 		  NIPQUAD(ciph->daddr), ntohs(pptr[1]));
 
-	/*
-	 * Send the mangled packet immediately here
-	 */
-	if (!(rt = __ip_vs_get_out_rt(cp, RT_TOS(iph->tos))))
-		goto tx_error_icmp;
-
-	/* MTU checking, is it necessary here??? */
-	mtu = rt->u.dst.pmtu;
-	if ((skb->len > mtu) && (iph->frag_off&__constant_htons(IP_DF))) {
-		ip_rt_put(rt);
-		icmp_send(skb, ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED, htonl(mtu));
-		IP_VS_DBG_RL("ip_vs_in_icmp(): frag needed\n");
-		goto tx_error;
-	}
-
-	/* drop old route */
-	dst_release(skb->dst);
-	skb->dst = &rt->u.dst;
-
 #ifdef CONFIG_NETFILTER_DEBUG
 	skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
@@ -1002,9 +1003,9 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 	ip_vs_conn_put(cp);
 	return NF_STOLEN;
 
- tx_error_icmp:
+  tx_error_icmp:
 	dst_link_failure(skb);
- tx_error:
+  tx_error:
 	dev_kfree_skb(skb);
 	ip_vs_conn_put(cp);
 	return NF_STOLEN;
