@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.28 2002/04/03 14:19:01 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.29 2002/05/30 13:00:20 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1594,7 +1594,7 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
 			       "   Total Incoming Outgoing         Incoming         Outgoing",
 			       "   Conns  Packets  Packets            Bytes            Bytes");
 
-		spin_lock(&ip_vs_stats.lock);
+		spin_lock_bh(&ip_vs_stats.lock);
 		sprintf(temp, "%8X %8X %8X %8X%08X %8X%08X",
 			ip_vs_stats.conns,
 			ip_vs_stats.inpkts,
@@ -1616,7 +1616,7 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
 			ip_vs_stats.outbps);
 		len += sprintf(buf+len, "%-63s\n", temp);
 
-		spin_unlock(&ip_vs_stats.lock);
+		spin_unlock_bh(&ip_vs_stats.lock);
 	}
 
 	*start = buf+len-(pos-offset);          /* Start of wanted data */
